@@ -1,12 +1,13 @@
-#!/usr/bin/env python3
+# Part of Spatial Math Toolbox for Python
+# Copyright (c) 2000 Peter Corke
+# MIT Licence, see details in top-level file: LICENCE
 
 # pylint: disable=invalid-name
 
 import sys
 import math
 import numpy as np
-from spatialmath import base as tr
-from spatialmath.base import argcheck
+from spatialmath import base
 
 _eps = np.finfo(np.float64).eps
 
@@ -16,11 +17,17 @@ def eye():
     Create an identity quaternion
 
     :return: an identity quaternion
-    :rtype: numpy.ndarray, shape=(4,)
+    :rtype: ndarray(4)
 
     Creates an identity quaternion, with the scalar part equal to one, and
     a zero vector value.
 
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import eye, qprint
+        >>> q = eye()
+        >>> qprint(q)
+
     """
     return np.r_[1, 0, 0, 0]
 
@@ -29,16 +36,21 @@ def pure(v):
     """
     Create a pure quaternion
 
-    :arg v: vector from a 3-vector
-    :type v: array_like
+    :arg v: 3D vector
+    :type v: array_like(3)
     :return: pure quaternion
-    :rtype: numpy.ndarray, shape=(4,)
+    :rtype: ndarray(4)
 
     Creates a pure quaternion, with a zero scalar value and the vector part
     equal to the passed vector value.
 
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import pure, qprint
+        >>> q = pure([1, 2, 3])
+        >>> qprint(q)
     """
-    v = argcheck.getvector(v, 3)
+    v = base.getvector(v, 3)
     return np.r_[0, v]
 
 
@@ -46,18 +58,24 @@ def qnorm(q):
     r"""
     Norm of a quaternion
 
-    :arg q: input quaternion as a 4-vector
-    :type v: : array_like
+    :arg q: quaternion
+    :type v: : array_like(4)
     :return: norm of the quaternion
     :rtype: float
 
     Returns the norm, length or magnitude of the input quaternion which is
-    :math:`\sqrt{s^2 + v_x^2 + v_y^2 + v_z^2}`
+    :math:`(s^2 + v_x^2 + v_y^2 + v_z^2}^{1/2}`
+
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import qnorm
+        >>> q = qnorm([1, 2, 3, 4])
+        >>> print(q)
 
     :seealso: unit
 
     """
-    q = argcheck.getvector(q, 4)
+    q = base.getvector(q, 4)
     return np.linalg.norm(q)
 
 
@@ -65,50 +83,69 @@ def unit(q, tol=10):
     """
     Create a unit quaternion
 
-    :arg v: quaterion as a 4-vector
-    :type v: array_like
+    :arg v: quaterion
+    :type v: array_like(4)
     :return: a pure quaternion
-    :rtype: numpy.ndarray, shape=(4,)
+    :rtype: ndarray(4)
+    :raises ValueError: quaternion has (near) zero norm
 
     Creates a unit quaternion, with unit norm, by scaling the input quaternion.
 
-    .. seealso:: norm
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import unit, qprint
+        >>> q = unit([1, 2, 3, 4])
+        >>> qprint(q)
+
+    .. note:: If the quaternion norm is less than ``tol * eps`` an exception is
+              raised.
+
+    :seealso: norm
     """
-    q = argcheck.getvector(q, 4)
+    q = base.getvector(q, 4)
     nm = np.linalg.norm(q)
-    assert abs(nm) > tol * _eps, 'cannot normalize (near) zero length quaternion'
+    if abs(nm) < tol * _eps:
+        raise ValueError("cannot normalize (near) zero length quaternion")
     return q / nm
 
 
-def isunit(q, tol=10):
+def isunit(q, tol=100):
     """
     Test if quaternion has unit length
 
-    :param v: quaternion as a 4-vector
-    :type v: array_like
+    :param v: quaternion
+    :type v: array_like(4)
     :param tol: tolerance in units of eps
     :type tol: float
     :return: whether quaternion has unit length
     :rtype: bool
 
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import eye, pure, isunit
+        >>> q = eye()
+        >>> isunit(q)
+        >>> q = pure([1, 2, 3])
+        >>> isunit(q)
+
     :seealso: unit
     """
-    return tr.iszerovec(q, tol=tol)
+    return base.iszerovec(q, tol=tol)
 
 
 def isequal(q1, q2, tol=100, unitq=False):
     """
     Test if quaternions are equal
 
-    :param q1: quaternion as a 4-vector
-    :type q1: array_like
-    :param q2: quaternion as a 4-vector
-    :type q2: array_like
+    :param q1: quaternion
+    :type q1: array_like(4)
+    :param q2: quaternion
+    :type q2: array_like(4)
     :param unitq: quaternions are unit quaternions
     :type unitq: bool
     :param tol: tolerance in units of eps
     :type tol: float
-    :return: whether quaternion has unit length
+    :return: whether quaternions are equal
     :rtype: bool
 
     Tests if two quaternions are equal.
@@ -116,9 +153,17 @@ def isequal(q1, q2, tol=100, unitq=False):
     For unit-quaternions ``unitq=True`` the double mapping is taken into account,
     that is ``q`` and ``-q`` represent the same orientation and ``isequal(q, -q, unitq=True)`` will
     return ``True``.
+
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import isequal
+        >>> q1 = [1, 2, 3, 4]
+        >>> q2 = [-1, -2, -3, -4]
+        >>> isequal(q1, q2)
+        >>> isequal(q1, q2, unitq=True)
     """
-    q1 = argcheck.getvector(q1, 4)
-    q2 = argcheck.getvector(q2, 4)
+    q1 = base.getvector(q1, 4)
+    q2 = base.getvector(q2, 4)
 
     if unitq:
         return (np.sum(np.abs(q1 - q2)) < tol * _eps) or (np.sum(np.abs(q1 + q2)) < tol * _eps)
@@ -130,21 +175,30 @@ def q2v(q):
     """
     Convert unit-quaternion to 3-vector
 
-    :arg q: unit-quaternion as a 4-vector
-    :type v: array_like
+    :arg q: unit-quaternion
+    :type v: array_like(4)
     :return: a unique 3-vector
-    :rtype: numpy.ndarray, shape=(3,)
+    :rtype: ndarray(3)
 
     Returns a unique 3-vector representing the input unit-quaternion. The sign
     of the scalar part is made positive, if necessary by multiplying the
     entire quaternion by -1, then the vector part is taken.
 
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import q2v
+        >>> from math import sqrt
+        >>> q = [1 / sqrt(2), 0, 1 / sqrt(2), 0]
+        >>> print(q2v(q))
+        >>> q = [-1 / sqrt(2), 0, 1 / sqrt(2), 0]
+        >>> print(q2v(q))
+        
     .. warning:: There is no check that the passed value is a unit-quaternion.
 
-    .. seealso:: v2q
+    :seealso: :func:`~v2q`
 
     """
-    q = argcheck.getvector(q, 4)
+    q = base.getvector(q, 4)
     if q[0] >= 0:
         return q[1:4]
     else:
@@ -155,17 +209,29 @@ def v2q(v):
     r"""
     Convert 3-vector to unit-quaternion
 
-    :arg v: vector part of unit quaternion, a 3-vector
-    :type v: array_like
+    :arg v: vector part of unit quaternion
+    :type v: array_like(3)
     :return: a unit quaternion
-    :rtype: numpy.ndarray, shape=(4,)
+    :rtype: ndarray(4)
 
     Returns a unit-quaternion reconsituted from just its vector part.  Assumes
     that the scalar part was positive, so :math:`s = \sqrt{1-||v||}`.
 
-    .. seealso:: q2v
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import v2q, qprint
+        >>> from math import sqrt
+        >>> v = [0, 1 / sqrt(2), 0]
+        >>> qprint(v2q(v))
+        >>> v = [0, -1 / sqrt(2), 0]
+        >>> qprint(v2q(v))
+
+    .. warning:: There is no check that the value is the vector part of
+                 a unit-quaternion, and this can lead to a math domain error.
+
+    :seealso: :func:`q2v`
     """
-    v = argcheck.getvector(v, 3)
+    v = base.getvector(v, 3)
     s = math.sqrt(1 - np.sum(v**2))
     return np.r_[s, v]
 
@@ -174,21 +240,28 @@ def qqmul(q1, q2):
     """
     Quaternion multiplication
 
-    :arg q0: left-hand quaternion as a 4-vector
-    :type q0: : array_like
-    :arg q1: right-hand quaternion as a 4-vector
-    :type q1: array_like
+    :arg q0: left-hand quaternion
+    :type q0: : array_like(4)
+    :arg q1: right-hand quaternion
+    :type q1: array_like(4)
     :return: quaternion product
-    :rtype: numpy.ndarray, shape=(4,)
+    :rtype: ndarray(4)
 
     This is the quaternion or Hamilton product.  If both operands are unit-quaternions then
     the product will be a unit-quaternion.
 
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import qqmul
+        >>> q1 = [1, 2, 3, 4]
+        >>> q2 = [5, 6, 7, 8]
+        >>> qqmul(q1, q2)    # conventional Hamilton product
+
     :seealso: qvmul, inner, vvmul
 
     """
-    q1 = argcheck.getvector(q1, 4)
-    q2 = argcheck.getvector(q2, 4)
+    q1 = base.getvector(q1, 4)
+    q2 = base.getvector(q2, 4)
     s1 = q1[0]
     v1 = q1[1:4]
     s2 = q2[0]
@@ -199,23 +272,37 @@ def qqmul(q1, q2):
 
 def inner(q1, q2):
     """
-    Quaternion innert product
+    Quaternion inner product
 
-    :arg q0: quaternion as a 4-vector
-    :type q0: : array_like
-    :arg q1: uaternion as a 4-vector
-    :type q1: array_like
+    :arg q0: quaternion 
+    :type q0: : array_like(4)
+    :arg q1: uaternion
+    :type q1: array_like(4)
     :return: inner product
-    :rtype: numpy.ndarray, shape=(4,)
+    :rtype: ndarray(4)
 
     This is the inner or dot product of two quaternions, it is the sum of the element-wise
     product.
 
+    - The inner product ``inner(q, q)`` is the square of the norm of ``q``.
+    - If ``q0`` and ``q1`` are unit quaternions then the inner product is the
+      cosine of the angle between the two orientations.
+
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import inner
+        >>> from math import sqrt, acos, pi
+        >>> q1 = [1, 2, 3, 4]
+        >>> inner(q1, q1)                      # square of the norm
+        >>> q1 = [1/sqrt(2), 1/sqrt(2), 0, 0]  # 90deg rotation about x-axis
+        >>> q2 = [1/sqrt(2), 0, 1/sqrt(2), 0]  # 90deg rotation about y-axis
+        >>> acos(inner(q1, q2)) * 180 / pi     # angle between q1 and q2
+
     :seealso: qvmul
 
     """
-    q1 = argcheck.getvector(q1, 4)
-    q2 = argcheck.getvector(q2, 4)
+    q1 = base.getvector(q1, 4)
+    q2 = base.getvector(q2, 4)
 
     return np.dot(q1, q2)
 
@@ -224,22 +311,29 @@ def qvmul(q, v):
     """
     Vector rotation
 
-    :arg q: unit-quaternion as a 4-vector
-    :type q: array_like
+    :arg q: unit-quaternion
+    :type q: array_like(4)
     :arg v: 3-vector to be rotated
-    :type v: list, tuple, numpy.ndarray
+    :type v: array_like(3)
     :return: rotated 3-vector
-    :rtype: numpy.ndarray, shape=(3,)
+    :rtype: ndarray(3)
 
     The vector `v` is rotated about the origin by the SO(3) equivalent of the unit
     quaternion.
 
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import qvmul
+        >>> from math import sqrt
+        >>> q = [1/sqrt(2), 1/sqrt(2), 0, 0]  # 90deg rotation about x-axis
+        >>> qvmul(q, [1, 2, 3])              # rotated vector
+
     .. warning:: There is no check that the passed value is a unit-quaternions.
 
     :seealso: qvmul
     """
-    q = argcheck.getvector(q, 4)
-    v = argcheck.getvector(v, 3)
+    q = base.getvector(q, 4)
+    v = base.getvector(v, 3)
     qv = qqmul(q, qqmul(pure(v), conj(q)))
     return qv[1:4]
 
@@ -249,18 +343,29 @@ def vvmul(qa, qb):
     Quaternion multiplication
 
 
-    :arg qa: left-hand quaternion as a 3-vector
-    :type qa: : array_like
-    :arg qb: right-hand quaternion as a 3-vector
-    :type qb: array_like
+    :arg qa: left-hand quaternion
+    :type qa: : array_like(3)
+    :arg qb: right-hand quaternion
+    :type qb: array_like(3)
     :return: quaternion product
-    :rtype: numpy.ndarray, shape=(3,)
+    :rtype: ndarray(3)
 
     This is the quaternion or Hamilton product of unit-quaternions defined only
     by their vector components.  The product will be a unit-quaternion, defined only
     by its vector component.
 
-    :seealso: qvmul, inner
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import vvmul, v2q, q2v, qqmul, qprint
+        >>> from math import sqrt
+        >>> q1 = [1/sqrt(2), 1/sqrt(2), 0, 0]  # 90deg rotation about x-axis
+        >>> q2 = [1/sqrt(2), 0, 1/sqrt(2), 0]  # 90deg rotation about y-axis
+        >>> qprint(qqmul(q1, q2))              # normal Hamilton product
+        >>> v1 = q2v(q1); v2 = q2v(q2)
+        >>> vp = vvmul(v1, v2)                 # product using 3-vectors
+        >>> qprint(v2q(vp))                    # same answer as Hamilton product
+
+    :seealso: :func:`q2v`, :func:`v2q`, :func:`qvmul`
     """
     t6 = math.sqrt(1.0 - np.sum(qa**2))
     t11 = math.sqrt(1.0 - np.sum(qb**2))
@@ -271,23 +376,35 @@ def qpow(q, power):
     """
     Raise quaternion to a power
 
-    :arg q: quaternion as a 4-vector
-    :type v: array_like
+    :arg q: quaternion
+    :type v: array_like(4)
     :arg power: exponent
     :type power: int
     :return: input quaternion raised to the specified power
-    :rtype: numpy.ndarray, shape=(4,)
+    :rtype: ndarray(4)
+    :raises ValueError: if exponent is non integer
 
     Raises a quaternion to the specified power using repeated multiplication.
 
-    Notes:
+    .. runblock:: pycon
 
-    - power must be an integer
-    - power can be negative, in which case the conjugate is taken
+        >>> from spatialmath.base import qpow, qqmul, qprint
+        >>> q = [1, 2, 3, 4]
+        >>> qprint(qqmul(q, q))
+        >>> qprint(qpow(q, 2))
+        >>> qprint(qpow(q, -2)) # conjugate of above
 
+    .. note:
+
+        - Power must be an integer
+        - Power can be negative, in which case the conjugate is taken
+
+    :seealso: :func:`qqmul`
+    :SymPy: supported for ``q`` but not ``power``.
     """
-    q = argcheck.getvector(q, 4)
-    assert isinstance(power, int), "Power must be an integer"
+    q = base.getvector(q, 4)
+    if not isinstance(power, int):
+        raise ValueError("Power must be an integer")
     qr = eye()
     for _ in range(0, abs(power)):
         qr = qqmul(qr, q)
@@ -302,15 +419,22 @@ def conj(q):
     """
     Quaternion conjugate
 
-    :arg q: quaternion as a 4-vector
-    :type v: array_like
+    :arg q: quaternion
+    :type v: array_like(4)
     :return: conjugate of input quaternion
-    :rtype: numpy.ndarray, shape=(4,)
+    :rtype: ndarray(4)
 
     Conjugate of quaternion, the vector part is negated.
 
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import conj, qprint
+        >>> q = [1, 2, 3, 4]
+        >>> qprint(conj(q))
+
+    :SymPy: supported
     """
-    q = argcheck.getvector(q, 4)
+    q = base.getvector(q, 4)
     return np.r_[q[0], -q[1:4]]
 
 
@@ -318,19 +442,25 @@ def q2r(q):
     """
     Convert unit-quaternion to SO(3) rotation matrix
 
-    :arg q: unit-quaternion as a 4-vector
-    :type v: array_like
+    :arg q: unit-quaternion
+    :type v: array_like(4)
     :return: corresponding SO(3) rotation matrix
-    :rtype: numpy.ndarray, shape=(3,3)
+    :rtype: ndarray(3,3)
 
     Returns an SO(3) rotation matrix corresponding to this unit-quaternion.
 
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import q2r
+        >>> q = [0, 0, 1, 0]  # rotation of 180deg about y-axis
+        >>> print(q2r(q))
+
     .. warning:: There is no check that the passed value is a unit-quaternion.
 
-    :seealso: r2q
+    :seealso: :func:`r2q`
 
     """
-    q = argcheck.getvector(q, 4)
+    q = base.getvector(q, 4)
     s = q[0]
     x = q[1]
     y = q[2]
@@ -344,25 +474,31 @@ def r2q(R, check=False, tol=100):
     """
     Convert SO(3) rotation matrix to unit-quaternion
 
-    :arg R: rotation matrix
-    :type R: numpy.ndarray, shape=(3,3)
+    :arg R: SO(3) rotation matrix
+    :type R: ndarray(3,3)
     :param check: check validity of rotation matrix, default False
     :type check: bool
     :param tol: tolerance in units of eps
     :type tol: float
     :return: unit-quaternion
-    :rtype: numpy.ndarray, shape=(3,)
+    :rtype: ndarray(4)
+    :raises ValueError: for non SO(3) argument
 
     Returns a unit-quaternion corresponding to the input SO(3) rotation matrix.
 
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import r2q, qprint, rotx
+        >>> R = rotx(90, 'deg') # rotation of 90deg about x-axis
+        >>> print(R)
+        >>> qprint(r2q(R))
+
     .. warning:: There is no check that the passed matrix is a valid rotation matrix.
 
-    :seealso: q2r
-
+    :seealso: :func:`q2r`
     """
-    assert R.shape == (3, 3) and tr.isR(R), "Argument must be 3x3 rotation matrix"
-    if check:
-        assert tr.isR(R, tol=tol), "Argument must be a valid SO(3) matrix"
+    if not base.isrot(R, check=check, tol=tol):
+        raise ValueError("Argument must be a valid SO(3) matrix")
 
     qs = math.sqrt(max(0, np.trace(R) + 1)) / 2.0
     kx = R[2, 1] - R[1, 2]  # Oz - Ay
@@ -406,16 +542,17 @@ def slerp(q0, q1, s, shortest=False):
     """
     Quaternion conjugate
 
-    :arg q0: initial unit quaternion as a 4-vector
-    :type q0: array_like
-    :arg q1: final unit quaternion as a 4-vector
-    :type q1: array_like
+    :arg q0: initial unit quaternion
+    :type q0: array_like(4)
+    :arg q1: final unit quaternion
+    :type q1: array_like(4)
     :arg s: interpolation coefficient in the range [0,1]
     :type s: float
     :arg shortest: choose shortest distance [default False]
     :type shortest: bool
     :return: interpolated unit-quaternion
-    :rtype: numpy.ndarray, shape=(4,)
+    :rtype: ndarray(4)
+    :raises ValueError: s is outside interval [0, 1]
 
     An interpolated quaternion between ``q0`` when ``s`` = 0 to ``q1`` when ``s`` = 1.
 
@@ -426,12 +563,23 @@ def slerp(q0, q1, s, shortest=False):
     For large rotations the path may be the *long way around* the circle,
     the option ``'shortest'`` ensures always the shortest path.
 
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import slerp, qprint
+        >>> from math import sqrt
+        >>> q0 = [1/sqrt(2), 1/sqrt(2), 0, 0]  # 90deg rotation about x-axis
+        >>> q1 = [1/sqrt(2), 0, 1/sqrt(2), 0]  # 90deg rotation about y-axis
+        >>> qprint(slerp(q0, q1, 0))           # this is q0
+        >>> qprint(slerp(q0, q1, 1))           # this is q1
+        >>> qprint(slerp(q0, q1, 0.5))         # this is in "half way" between
+
     .. warning:: There is no check that the passed values are unit-quaternions.
 
     """
-    assert 0 <= s <= 1, 's must be in the interval [0,1]'
-    q0 = argcheck.getvector(q0, 4)
-    q1 = argcheck.getvector(q1, 4)
+    if not 0 <= s <= 1:
+        raise ValueError("s must be in the interval [0,1]")
+    q0 = base.getvector(q0, 4)
+    q1 = base.getvector(q1, 4)
 
     if s == 0:
         return q0
@@ -464,10 +612,15 @@ def rand():
     Random unit-quaternion
 
     :return: random unit-quaternion
-    :rtype: numpy.ndarray, shape=(4,)
+    :rtype: ndarray(4)
 
     Computes a uniformly distributed random unit-quaternion which can be
     considered equivalent to a random SO(3) rotation.
+
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import rand, qprint
+        >>> qprint(rand())
     """
     u = np.random.uniform(low=0, high=1, size=3)  # get 3 random numbers in [0,1]
     return np.r_[
@@ -481,19 +634,30 @@ def matrix(q):
     """
     Convert to 4x4 matrix equivalent
 
-    :arg q: quaternion as a 4-vector
-    :type v: array_like
+    :arg q: quaternion
+    :type v: array_like(4)
     :return: equivalent matrix
-    :rtype: numpy.ndarray, shape=(4,4)
+    :rtype: ndarray(4)
 
     Hamilton multiplication between two quaternions can be considered as a
     matrix-vector product, the left-hand quaternion is represented by an
     equivalent 4x4 matrix and the right-hand quaternion as 4x1 column vector.
 
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import matrix, qqmul, qprint
+        >>> q1 = [1, 2, 3, 4]
+        >>> q2 = [5, 6, 7, 8]
+        >>> qqmul(q1, q2)    # conventional Hamilton product
+        >>> m = matrix(q1)
+        >>> print(m)
+        >>> v = m @ np.array(q2)
+        >>> print(v)
+
     :seealso: qqmul
 
     """
-    q = argcheck.getvector(q, 4)
+    q = base.getvector(q, 4)
     s = q[0]
     x = q[1]
     y = q[2]
@@ -508,23 +672,30 @@ def dot(q, w):
     """
     Rate of change of unit-quaternion
 
-    :arg q0: unit-quaternion as a 4-vector
-    :type q0: array_like
-    :arg w: angular velocity in world frame as a 3-vector
-    :type w: array_like
+    :arg q0: unit-quaternion
+    :type q0: array_like(4)
+    :arg w: 3D angular velocity in world frame
+    :type w: array_like(3)
     :return: rate of change of unit quaternion
-    :rtype: numpy.ndarray, shape=(4,)
+    :rtype: ndarray(4)
 
     ``dot(q, w)`` is the rate of change of the elements of the unit quaternion ``q``
     which represents the orientation of a body frame with angular velocity ``w`` in
     the world frame.
 
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import dot, qprint
+        >>> from math import sqrt
+        >>> q = [1/sqrt(2), 1/sqrt(2), 0, 0]   # 90deg rotation about x-axis
+        >>> dot(q, [1, 2, 3])
+
     .. warning:: There is no check that the passed values are unit-quaternions.
 
     """
-    q = argcheck.getvector(q, 4)
-    w = argcheck.getvector(w, 3)
-    E = q[0] * (np.eye(3, 3)) - tr.skew(q[1:4])
+    q = base.getvector(q, 4)
+    w = base.getvector(w, 3)
+    E = q[0] * (np.eye(3, 3)) - base.skew(q[1:4])
     return 0.5 * np.r_[-np.dot(q[1:4], w), E@w]
 
 
@@ -532,23 +703,30 @@ def dotb(q, w):
     """
     Rate of change of unit-quaternion
 
-    :arg q0: unit-quaternion as a 4-vector
-    :type q0: array_like
-    :arg w: angular velocity in body frame as a 3-vector
-    :type w: array_like
+    :arg q0: unit-quaternion
+    :type q0: array_like(4)
+    :arg w: 3D angular velocity in body frame
+    :type w: array_like(3)
     :return: rate of change of unit quaternion
-    :rtype: numpy.ndarray, shape=(4,)
+    :rtype: ndarray(4)
 
     ``dotb(q, w)`` is the rate of change of the elements of the unit quaternion ``q``
     which represents the orientation of a body frame with angular velocity ``w`` in
     the body frame.
 
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import dotb, qprint
+        >>> from math import sqrt
+        >>> q = [1/sqrt(2), 1/sqrt(2), 0, 0]   # 90deg rotation about x-axis
+        >>> dotb(q, [1, 2, 3])
+
     .. warning:: There is no check that the passed values are unit-quaternions.
 
     """
-    q = argcheck.getvector(q, 4)
-    w = argcheck.getvector(w, 3)
-    E = q[0] * (np.eye(3, 3)) + tr.skew(q[1:4])
+    q = base.getvector(q, 4)
+    w = base.getvector(w, 3)
+    E = q[0] * (np.eye(3, 3)) + base.skew(q[1:4])
     return 0.5 * np.r_[-np.dot(q[1:4], w), E@w]
 
 
@@ -556,10 +734,10 @@ def angle(q1, q2):
     """
     Angle between two unit-quaternions
 
-    :arg q0: unit-quaternion as a 4-vector
-    :type q0: array_like
-    :arg q1: unit-quaternion as a 4-vector
-    :type q1: array_like
+    :arg q0: unit-quaternion
+    :type q0: array_like(4)
+    :arg q1: unit-quaternion
+    :type q1: array_like(4)
     :return: angle between the rotations [radians]
     :rtype: float
 
@@ -567,28 +745,38 @@ def angle(q1, q2):
     frame, then the angle is the smallest rotation required about a fixed
     axis, to rotate the first frame into the second.
 
-    References:  Metrics for 3D rotations: comparison and analysis,
-    Du Q. Huynh, % J.Math Imaging Vis. DOFI 10.1007/s10851-009-0161-2.
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import angle
+        >>> from math import sqrt
+        >>> q1 = [1/sqrt(2), 1/sqrt(2), 0, 0]    # 90deg rotation about x-axis
+        >>> q2 = [1/sqrt(2), 0, 1/sqrt(2), 0]    # 90deg rotation about y-axis
+        >>> angle(q1, q2)
+
+    :References:  
+    
+    - Metrics for 3D rotations: comparison and analysis,
+      Du Q. Huynh, % J.Math Imaging Vis. DOFI 10.1007/s10851-009-0161-2.
 
     .. warning:: There is no check that the passed values are unit-quaternions.
 
     """
     # TODO different methods
 
-    q1 = argcheck.getvector(q1, 4)
-    q2 = argcheck.getvector(q2, 4)
-    return 2.0 * math.atan2(tr.norm(q1 - q2), tr.norm(q1 + q2))
+    q1 = base.getvector(q1, 4)
+    q2 = base.getvector(q2, 4)
+    return 2.0 * math.atan2(base.norm(q1 - q2), base.norm(q1 + q2))
 
 
-def qprint(q, delim=('<', '>'), fmt='%f', file=sys.stdout):
+def qprint(q, delim=('<', '>'), fmt='{: .4f}', file=sys.stdout):
     """
     Format a quaternion
 
-    :arg q: unit-quaternion as a 4-vector
-    :type q: array_like
+    :arg q: unit-quaternion
+    :type q: array_like(4)
     :arg delim: 2-list of delimeters [default ('<', '>')]
     :type delim: list or tuple of strings
-    :arg fmt: printf-style format soecifier [default '%f']
+    :arg fmt: printf-style format soecifier [default '{: .4f}']
     :type fmt: str
     :arg file: destination for formatted string [default sys.stdout]
     :type file: file object
@@ -606,10 +794,17 @@ def qprint(q, delim=('<', '>'), fmt='%f', file=sys.stdout):
 
     If `file=None` then a string is returned.
 
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import qprint, rand
+        >>> q = [1, 2, 3, 4]
+        >>> qprint(q)
+        >>> q = rand()   # a unit quaternion
+        >>> qprint(q, delim=('<<', '>>'))
     """
-    q = argcheck.getvector(q, 4)
-    template = "# %s #, #, # %s".replace('#', fmt)
-    s = template % (q[0], delim[0], q[1], q[2], q[3], delim[1])
+    q = base.getvector(q, 4)
+    template = "# {} #, #, # {}".replace('#', fmt)
+    s = template.format(q[0], delim[0], q[1], q[2], q[3], delim[1])
     if file:
         file.write(s + '\n')
     else:
