@@ -1,54 +1,23 @@
+# Part of Spatial Math Toolbox for Python
+# Copyright (c) 2000 Peter Corke
+# MIT Licence, see details in top-level file: LICENCE
+
 """
-This modules contains functions to create and transform rotation matrices
-and homogeneous tranformation matrices.
+This modules contains functions to create and transform SO(2) and SE(2) matrices,
+respectively 2D rotation matrices and homogeneous tranformation matrices.
 
 Vector arguments are what numpy refers to as ``array_like`` and can be a list,
 tuple, numpy array, numpy row vector or numpy column vector.
 
 """
 
-# This file is part of the SpatialMath toolbox for Python
-# https://github.com/petercorke/spatialmath-python
-#
-# MIT License
-#
-# Copyright (c) 1993-2020 Peter Corke
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in all
-# copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
-
-# Contributors:
-#
-#     1. Luis Fernando Lara Tobar and Peter Corke, 2008
-#     2. Josh Carrigg Hodson, Aditya Dua, Chee Ho Chan, 2017 (robopy)
-#     3. Peter Corke, 2020
-
 # pylint: disable=invalid-name
 
 import sys
 import math
 import numpy as np
 import scipy.linalg
-from spatialmath.base import argcheck
-from spatialmath.base import vectors as vec
-from spatialmath.base import transformsNd as trn
-from spatialmath.base import animate
-from spatialmath.base import symbolic as sym
+from spatialmath import base
 
 _eps = np.finfo(np.float64).eps
 
@@ -62,15 +31,21 @@ def rot2(theta, unit='rad'):
     :type theta: float
     :param unit: angular units: 'rad' [default], or 'deg'
     :type unit: str
-    :return: 2x2 rotation matrix
-    :rtype: numpy.ndarray, shape=(2,2)
+    :return: SO(2) rotation matrix
+    :rtype: ndarray(2,2)
 
-    - ``ROT2(THETA)`` is an SO(2) rotation matrix (2x2) representing a rotation of THETA radians.
-    - ``ROT2(THETA, 'deg')`` as above but THETA is in degrees.
+    - ``rot2(θ)`` is an SO(2) rotation matrix (2x2) representing a rotation of θ radians.
+    - ``rot2(θ, 'deg')`` as above but θ is in degrees.
+
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import *
+        >>> rot2(0.3)
+        >>> rot2(45, 'deg')
     """
-    theta = argcheck.getunit(theta, unit)
-    ct = sym.cos(theta)
-    st = sym.sin(theta)
+    theta = base.getunit(theta, unit)
+    ct = base.sym.cos(theta)
+    st = base.sym.sin(theta)
     R = np.array([
         [ct, -st],
         [st, ct]])
@@ -83,23 +58,30 @@ def trot2(theta, unit='rad', t=None):
     Create SE(2) pure rotation
 
     :param theta: rotation angle about X-axis
-    :type theta: float
+    :type θ: float
     :param unit: angular units: 'rad' [default], or 'deg'
     :type unit: str
-    :param t: translation 2-vector, defaults to [0,0]
-    :type t: array_like    :return: 3x3 homogeneous transformation matrix
-    :rtype: numpy.ndarray, shape=(3,3)
+    :param t: 2D translation vector, defaults to [0,0]
+    :type t: array_like(2)   
+    :return: 3x3 homogeneous transformation matrix
+    :rtype: ndarray(3,3)
 
-    - ``TROT2(THETA)`` is a homogeneous transformation (3x3) representing a rotation of
-      THETA radians.
-    - ``TROT2(THETA, 'deg')`` as above but THETA is in degrees.
+    - ``trot2(θ)`` is a homogeneous transformation (3x3) representing a rotation of
+      θ radians.
+    - ``trot2(θ, 'deg')`` as above but θ is in degrees.
 
-    Notes:
-    - Translational component is zero.
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import *
+        >>> trot2(0.3)
+        >>> trot2(45, 'deg', t=[1,2])
+
+    .. note:: By default, the translational component is zero but it can be 
+        set to a non-zero value.
     """
     T = np.pad(rot2(theta, unit), (0, 1), mode='constant')
     if t is not None:
-        T[:2, 2] = argcheck.getvector(t, 2, 'array')
+        T[:2, 2] = base.getvector(t, 2, 'array')
     T[2, 2] = 1.0
     return T
 
@@ -109,14 +91,15 @@ def transl2(x, y=None):
     """
     Create SE(2) pure translation, or extract translation from SE(2) matrix
 
+
+    **Create a translational SE(2) matrix**
+
     :param x: translation along X-axis
     :type x: float
     :param y: translation along Y-axis
     :type y: float
-    :return: homogeneous transform matrix or the translation elements of a homogeneous transform
-    :rtype: numpy.ndarray, shape=(3,3)
-
-    Create a translational SE(2) matrix:
+    :return: SE(2) transform matrix or the translation elements of a homogeneous transform
+    :rtype: ndarray(3,3)
 
     - ``T = transl2([X, Y])`` is an SE(2) homogeneous transform (3x3) representing a
       pure translation.
@@ -124,21 +107,46 @@ def transl2(x, y=None):
       list, dict, or a numpy array, row or column vector.
 
 
-    Extract the translational part of an SE(2) matrix:
+    .. runblock:: pycon
 
-    P = TRANSL2(T) is the translational part of a homogeneous transform as a
-    2-element numpy array.
+        >>> from spatialmath.base import *
+        >>> import numpy as np
+        >>> transl2(3, 4)
+        >>> transl2([3, 4])
+        >>> transl2(np.array([3, 4]))
+
+    **Extract the translational part of an SE(2) matrix**
+
+    :param x: SE(2) transform matrix
+    :type x: ndarray(3,3)
+    :return: translation elements of SE(2) matrix
+    :rtype: ndarray(2)
+
+    - ``t = transl2(T)`` is the translational part of the SE(3) matrix ``T`` as a
+      2-element NumPy array.
+
+
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import *
+        >>> import numpy as np
+        >>> T = np.array([[1, 0, 3], [0, 1, 4], [0, 0, 1]])
+        >>> transl2(T)
+        
+    .. note:: This function is compatible with the MATLAB version of the Toolbox.  It
+        is unusual/weird in doing two completely different things inside the one
+        function.
     """
 
     if np.isscalar(x):
         T = np.identity(3)
         T[:2, 2] = [x, y]
         return T
-    elif argcheck.isvector(x, 2):
+    elif base.isvector(x, 2):
         T = np.identity(3)
-        T[:2, 2] = argcheck.getvector(x, 2)
+        T[:2, 2] = base.getvector(x, 2)
         return T
-    elif argcheck.ismatrix(x, (3, 3)):
+    elif base.ismatrix(x, (3, 3)):
         return x[:2, 2]
     else:
         ValueError('bad argument')
@@ -148,21 +156,33 @@ def ishom2(T, check=False):
     """
     Test if matrix belongs to SE(2)
 
-    :param T: matrix to test
-    :type T: numpy.ndarray
+    :param T: SE(2) matrix to test
+    :type T: ndarray(3,3)
     :param check: check validity of rotation submatrix
     :type check: bool
     :return: whether matrix is an SE(2) homogeneous transformation matrix
     :rtype: bool
 
-    - ``ISHOM2(T)`` is True if the argument ``T`` is of dimension 3x3
-    - ``ISHOM2(T, check=True)`` as above, but also checks orthogonality of the
+    - ``ishom2(T)`` is True if the argument ``T`` is of dimension 3x3
+    - ``ishom2(T, check=True)`` as above, but also checks orthogonality of the
       rotation sub-matrix and validitity of the bottom row.
 
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import *
+        >>> import numpy as np
+        >>> T = np.array([[1, 0, 3], [0, 1, 4], [0, 0, 1]])
+        >>> ishom2(T)
+        >>> T = np.array([[1, 1, 3], [0, 1, 4], [0, 0, 1]]) # invalid SE(2)
+        >>> ishom2(T)  # a quick check says it is an SE(2)
+        >>> ishom2(T, check=True) # but if we check more carefully...
+        >>> R = np.array([[1, 0], [0, 1]])
+        >>> ishom2(R)
+
     :seealso: isR, isrot2, ishom, isvec
     """
     return isinstance(T, np.ndarray) and T.shape == (3, 3) \
-        and (not check or (trn.isR(T[:2, :2])
+        and (not check or (base.isR(T[:2, :2])
                            and np.all(T[2, :] == np.array([0, 0, 1]))))
 
 
@@ -170,20 +190,32 @@ def isrot2(R, check=False):
     """
     Test if matrix belongs to SO(2)
 
-    :param R: matrix to test
-    :type R: numpy.ndarray
+    :param R: SO(2) matrix to test
+    :type R: ndarray(3,3)
     :param check: check validity of rotation submatrix
     :type check: bool
     :return: whether matrix is an SO(2) rotation matrix
     :rtype: bool
 
-    - ``ISROT(R)`` is True if the argument ``R`` is of dimension 2x2
-    - ``ISROT(R, check=True)`` as above, but also checks orthogonality of the rotation matrix.
+    - ``isrot2(R)`` is True if the argument ``R`` is of dimension 2x2
+    - ``isrot2(R, check=True)`` as above, but also checks orthogonality of the rotation matrix.
+
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import *
+        >>> import numpy as np
+        >>> T = np.array([[1, 0, 3], [0, 1, 4], [0, 0, 1]])
+        >>> isrot2(T)
+        >>> R = np.array([[1, 0], [0, 1]])
+        >>> isrot2(R)
+        >>> R = np.array([[1, 1], [0, 1]])  # invalid SO(2)
+        >>> isrot2(R)  # a quick check says it is an SO(2)
+        >>> isrot2(R, check=True)  # but if we check more carefully...
 
     :seealso: isR, ishom2, isrot
     """
     return isinstance(R, np.ndarray) and R.shape == (2, 2) \
-        and (not check or trn.isR(R))
+        and (not check or base.isR(R))
 
 # ---------------------------------------------------------------------------------------#
 
@@ -192,15 +224,15 @@ def trlog2(T, check=True, twist=False):
     """
     Logarithm of SO(2) or SE(2) matrix
 
-    :param T: SO(2) or SE(2) matrix
-    :type T: numpy.ndarray, shape=(2,2) or (3,3)
+    :param T: SE(2) or SO(2) matrix
+    :type T: ndarray(3,3) or ndarray(2,2)
     :param check: check that matrix is valid
     :type check: bool
     :param twist: return a twist vector instead of matrix [default]
     :type twist: bool
     :return: logarithm
-    :rtype: numpy.ndarray, shape=(2,2) or (3,3)
-    :raises: ValueError
+    :rtype: ndarray(3,3) or ndarray(3); or ndarray(2,2) or ndarray(1)
+    :raises ValueError: bad argument
 
     An efficient closed-form solution of the matrix logarithm for arguments that
     are SO(2) or SE(2).
@@ -214,6 +246,14 @@ def trlog2(T, check=True, twist=False):
       equivalent vector from ``vexa()`` is the twist vector (6x1) comprising [v
       w].
 
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import *
+        >>> trlog2(trot2(0.3))
+        >>> trlog2(trot2(0.3), twist=True)
+        >>> trlog2(rot2(0.3))
+        >>> trlog2(rot2(0.3), twist=True)
+
     :seealso: :func:`~trexp`, :func:`~spatialmath.base.transformsNd.vex`,
               :func:`~spatialmath.base.transformsNd.vexa`
     """
@@ -221,7 +261,7 @@ def trlog2(T, check=True, twist=False):
     if ishom2(T, check=check):
         # SE(2) matrix
 
-        if trn.iseye(T):
+        if base.iseye(T):
             # is identity matrix
             if twist:
                 return np.zeros((3,))
@@ -229,14 +269,14 @@ def trlog2(T, check=True, twist=False):
                 return np.zeros((3, 3))
         else:
             if twist:
-                return trn.vexa(scipy.linalg.logm(T))
+                return base.vexa(scipy.linalg.logm(T))
             else:
                 return scipy.linalg.logm(T)
 
     elif isrot2(T, check=check):
         # SO(2) rotation matrix
         if twist:
-            return trn.vex(scipy.linalg.logm(T))
+            return base.vex(scipy.linalg.logm(T))
         else:
             return scipy.linalg.logm(T)
     else:
@@ -248,107 +288,124 @@ def trexp2(S, theta=None, check=True):
     """
     Exponential of so(2) or se(2) matrix
 
-    :param S: so(2), se(2) matrix or equivalent velctor
-    :type T: numpy.ndarray, shape=(2,2) or (3,3); array_like
+    :param S: se(2), so(2) matrix or equivalent velctor
+    :type T: ndarray(3,3) or ndarray(2,2)
     :param theta: motion
     :type theta: float
-    :return: 2x2 or 3x3 matrix exponential in SO(2) or SE(2)
-    :rtype: numpy.ndarray, shape=(2,2) or (3,3)
+    :return: matrix exponential in SE(2) or SO(2)
+    :rtype: ndarray(3,3) or ndarray(2,2)
+    :raises ValueError: bad argument
 
     An efficient closed-form solution of the matrix exponential for arguments
-    that are so(2) or se(2).
-
-    For so(2) the results is an SO(2) rotation matrix:
-
-    - ``trexp2(S)`` is the matrix exponential of the so(3) element ``S`` which is a 2x2
-      skew-symmetric matrix.
-    - ``trexp2(S, THETA)`` as above but for an so(3) motion of S*THETA, where ``S`` is
-      unit-norm skew-symmetric matrix representing a rotation axis and a rotation magnitude
-      given by ``THETA``.
-    - ``trexp2(W)`` is the matrix exponential of the so(2) element ``W`` expressed as
-      a 1-vector (array_like).
-    - ``trexp2(W, THETA)`` as above but for an so(3) motion of W*THETA where ``W`` is a
-      unit-norm vector representing a rotation axis and a rotation magnitude
-      given by ``THETA``. ``W`` is expressed as a 1-vector (array_like).
-
+    that are se(2) or so(2).
 
     For se(2) the results is an SE(2) homogeneous transformation matrix:
 
-    - ``trexp2(SIGMA)`` is the matrix exponential of the se(2) element ``SIGMA`` which is
+    - ``trexp2(Σ)`` is the matrix exponential of the se(2) element ``Σ`` which is
       a 3x3 augmented skew-symmetric matrix.
-    - ``trexp2(SIGMA, THETA)`` as above but for an se(3) motion of SIGMA*THETA, where ``SIGMA``
+    - ``trexp2(Σ, θ)`` as above but for an se(3) motion of Σθ, where ``Σ``
       must represent a unit-twist, ie. the rotational component is a unit-norm skew-symmetric
       matrix.
-    - ``trexp2(TW)`` is the matrix exponential of the se(3) element ``TW`` represented as
+    - ``trexp2(S)`` is the matrix exponential of the se(3) element ``S`` represented as
       a 3-vector which can be considered a screw motion.
-    - ``trexp2(TW, THETA)`` as above but for an se(2) motion of TW*THETA, where ``TW``
+    - ``trexp2(S, θ)`` as above but for an se(2) motion of Sθ, where ``S``
       must represent a unit-twist, ie. the rotational component is a unit-norm skew-symmetric
       matrix.
 
-     :seealso: trlog, trexp2
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import *
+        >>> trexp2(skew(1))
+        >>> trexp2(skew(1), 2)  # revolute unit twist
+        >>> trexp2(1)
+        >>> trexp2(1, 2)  # revolute unit twist
+
+    For so(2) the results is an SO(2) rotation matrix:
+
+    - ``trexp2(Ω)`` is the matrix exponential of the so(3) element ``Ω`` which is a 2x2
+      skew-symmetric matrix.
+    - ``trexp2(Ω, θ)`` as above but for an so(3) motion of Ωθ, where ``Ω`` is
+      unit-norm skew-symmetric matrix representing a rotation axis and a rotation magnitude
+      given by ``θ``.
+    - ``trexp2(ω)`` is the matrix exponential of the so(2) element ``ω`` expressed as
+      a 1-vector.
+    - ``trexp2(ω, θ)`` as above but for an so(3) motion of ωθ where ``ω`` is a
+      unit-norm vector representing a rotation axis and a rotation magnitude
+      given by ``θ``. ``ω`` is expressed as a 1-vector.
+
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import *
+        >>> trexp2(skewa([1, 2, 3]))
+        >>> trexp2(skewa([1, 0, 0]), 2)  # prismatic unit twist
+        >>> trexp2([1, 2, 3])
+        >>> trexp2([1, 0, 0], 2)
+
+    :seealso: trlog, trexp2
     """
 
-    if argcheck.ismatrix(S, (3, 3)) or argcheck.isvector(S, 3):
+    if base.ismatrix(S, (3, 3)) or base.isvector(S, 3):
         # se(2) case
-        if argcheck.ismatrix(S, (3, 3)):
+        if base.ismatrix(S, (3, 3)):
             # augmentented skew matrix
-            if check:
-                assert trn.isskewa(S), 'argument must be a valid se(2) element'
-            tw = trn.vexa(S)
+            if check and not base.isskewa(S):
+                raise ValueError("argument must be a valid se(2) element")
+            tw = base.vexa(S)
         else:
             # 3 vector
-            tw = argcheck.getvector(S)
+            tw = base.getvector(S)
 
-        if vec.iszerovec(tw):
+        if base.iszerovec(tw):
             return np.eye(3)
 
         if theta is None:
-            (tw, theta) = vec.unittwist2(tw)
-        else:
-            assert vec.isunittwist2(tw), 'If theta is specified S must be a unit twist'
+            (tw, theta) = base.unittwist2_norm(tw)
+        elif not base.isunittwist2(tw):
+            raise ValueError("If theta is specified S must be a unit twist")
 
         t = tw[0:2]
         w = tw[2]
 
-        R = trn.rodrigues(w, theta)
+        R = base.rodrigues(w, theta)
 
-        skw = trn.skew(w)
+        skw = base.skew(w)
         V = np.eye(2) * theta + (1.0 - math.cos(theta)) * skw + (theta - math.sin(theta)) * skw @ skw
 
-        return trn.rt2tr(R, V@t)
+        return base.rt2tr(R, V@t)
 
-    elif argcheck.ismatrix(S, (2, 2)) or argcheck.isvector(S, 1):
+    elif base.ismatrix(S, (2, 2)) or base.isvector(S, 1):
         # so(2) case
-        if argcheck.ismatrix(S, (2, 2)):
+        if base.ismatrix(S, (2, 2)):
             # skew symmetric matrix
-            if check:
-                assert trn.isskew(S), 'argument must be a valid so(2) element'
-            w = trn.vex(S)
+            if check and not base.isskew(S):
+                raise ValueError("argument must be a valid so(2) element")
+            w = base.vex(S)
         else:
             # 1 vector
-            w = argcheck.getvector(S)
+            w = base.getvector(S)
 
-        if theta is not None:
-            assert vec.isunitvec(w), 'If theta is specified S must be a unit twist'
+        if theta is not None and not base.isunitvec(w):
+            raise ValueError("If theta is specified S must be a unit twist")
 
         # do Rodrigues' formula for rotation
-        return trn.rodrigues(w, theta)
+        return base.rodrigues(w, theta)
     else:
         raise ValueError(" First argument must be SO(2), 1-vector, SE(2) or 3-vector")
 
 
 def trinterp2(start, end, s=None):
     """
-    Interpolate SE(2) matrices
+    Interpolate SE(2) or SO(2) matrices
 
-    :param start: initial SO(2) or SE(2) matrix value when s=0, if None then identity is used
-    :type T1: np.ndarray, shape=(2,2), (3,3) or None
-    :param end: final SO(2) or SE(2) matrix, value when s=1
-    :type T0: np.ndarray, shape=(2,2), (3,3)
+    :param start: initial SE(2) or SO(2) matrix value when s=0, if None then identity is used
+    :type start: ndarray(3,3) or ndarray(2,2) or None
+    :param end: final SE(2) or SO(2) matrix, value when s=1
+    :type end: ndarray(3,3) or ndarray(2,2)
     :param s: interpolation coefficient, range 0 to 1
     :type s: float
-    :return: SO(2) or SE(2) matrix
-    :rtype: np.ndarray, shape=(2,2), (3,3)
+    :return: interpolated SE(2) or SO(2) matrix value
+    :rtype: ndarray(3,3) or ndarray(2,2)
+    :raises ValueError: bad arguments
 
     - ``trinterp2(None, T, S)`` is a homogeneous transform (3x3) interpolated
       between identity when S=0 and T (3x3) when S=1.
@@ -359,15 +416,24 @@ def trinterp2(start, end, s=None):
     - ``trinterp2(R0, R1, S)`` as above but interpolated
       between R0 (2x2) when S=0 and R1 (2x2) when S=1.
 
-    Notes:
+    .. note:: Rotation angle is linearly interpolated.
 
-    - Rotation angle is linearly interpolated.
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import *
+        >>> T1 = transl2(1, 2)
+        >>> T2 = transl2(3, 4)
+        >>> trinterp2(T1, T2, 0)
+        >>> trinterp2(T1, T2, 1)
+        >>> trinterp2(T1, T2, 0.5)
+        >>> trinterp2(None, T2, 0)
+        >>> trinterp2(None, T2, 1)
+        >>> trinterp2(None, T2, 0.5)
 
     :seealso: :func:`~spatialmath.base.transforms3d.trinterp`
 
-    %## 2d homogeneous trajectory
     """
-    if argcheck.ismatrix(end, (2, 2)):
+    if base.ismatrix(end, (2, 2)):
         # SO(2) case
         if start is None:
             #	TRINTERP2(T, s)
@@ -377,7 +443,8 @@ def trinterp2(start, end, s=None):
             th = s * th0
         else:
             #	TRINTERP2(T1, start= s)
-            assert start.shape == end.shape, 'start and end matrices must be same shape'
+            if start.shape != end.shape:
+                raise ValueError("start and end matrices must be same shape")
 
             th0 = math.atan2(start[1, 0], start[0, 0])
             th1 = math.atan2(end[1, 0], end[0, 0])
@@ -385,7 +452,7 @@ def trinterp2(start, end, s=None):
             th = th0 * (1 - s) + s * th1
 
         return rot2(th)
-    elif argcheck.ismatrix(end, (3, 3)):
+    elif base.ismatrix(end, (3, 3)):
         if start is None:
             #	TRINTERP2(T, s)
 
@@ -396,7 +463,8 @@ def trinterp2(start, end, s=None):
             pr = s * p0
         else:
             #	TRINTERP2(T0, T1, s)
-            assert start.shape == end.shape, 'both matrices must be same shape'
+            if start.shape != end.shape:
+                raise ValueError("both matrices must be same shape")
 
             th0 = math.atan2(start[1, 0], start[0, 0])
             th1 = math.atan2(end[1, 0], end[0, 0])
@@ -407,21 +475,21 @@ def trinterp2(start, end, s=None):
             pr = p0 * (1 - s) + s * p1
             th = th0 * (1 - s) + s * th1
 
-        return trn.rt2tr(rot2(th), pr)
+        return base.rt2tr(rot2(th), pr)
     else:
         return ValueError('Argument must be SO(2) or SE(2)')
 
 
 def trprint2(T, label=None, file=sys.stdout, fmt='{:8.2g}', unit='deg'):
     """
-    Compact display of SO(2) or SE(2) matrices
+    Compact display of SE(2) or SO(2) matrices
 
     :param T: matrix to format
-    :type T: numpy.ndarray, shape=(2,2) or (3,3)
+    :type T: ndarray(3,3) or ndarray(2,2)
     :param label: text label to put at start of line
     :type label: str
     :param file: file to write formatted string to
-    :type file: str
+    :type file: file object
     :param fmt: conversion format for each number
     :type fmt: str
     :param unit: angular units: 'rad' [default], or 'deg'
@@ -435,18 +503,18 @@ def trprint2(T, label=None, file=sys.stdout, fmt='{:8.2g}', unit='deg'):
     - ``trprint2(R)`` displays the SO(2) rotation matrix in a compact
       single-line format and returns the string::
 
-        [LABEL:] THETA UNIT
+        [LABEL:] θ UNIT
 
     - ``trprint2(T)`` displays the SE(2) homogoneous transform in a compact
       single-line format and returns the string::
 
-        [LABEL:] [t=X, Y;] THETA UNIT
+        [LABEL:] [t=X, Y;] θ UNIT
 
-    Example::
+    .. runblock:: pycon
 
+        >>> from spatialmath.base import *
         >>> T = transl2(1,2) @ trot2(0.3)
-        >>> trprint2(a, file=None, label='T')
-        'T: t =        1,        2;       17 deg'
+        >>> trprint2(T, file=None, label='T')
 
     :seealso: trprint
     """
@@ -492,14 +560,14 @@ if _matplotlib_exists:
         """
         Plot a 2D coordinate frame
 
-        :param T: an SO(3) or SE(3) pose to be displayed as coordinate frame
-        :type: numpy.ndarray, shape=(2,2) or (3,3)
+        :param T: an SE(3) or SO(3) pose to be displayed as coordinate frame
+        :type: ndarray(3,3) or ndarray(2,2)
         :param axes: the axes to plot into, defaults to current axes
         :type axes: Axes3D reference
         :param block: run the GUI main loop until all windows are closed, default True
         :type block: bool
         :param dims: dimension of plot volume as [xmin, xmax, ymin, ymax]
-        :type dims: array_like
+        :type dims: array_like(4)
         :param color: color of the lines defining the frame
         :type color: str
         :param textcolor: color of text labels for the frame, default color of lines above
@@ -524,6 +592,7 @@ if _matplotlib_exists:
         :type d2: distance of frame label text from origin, default 0.05
         :return: axes containing the frame
         :rtype: AxesSubplot
+        :raises ValueError: bad argument
 
         Adds a 2D coordinate frame represented by the SO(2) or SE(2) matrix to the current axes.
 
@@ -544,9 +613,9 @@ if _matplotlib_exists:
 
         # check input types
         if isrot2(T, check=True):
-            T = trn.r2t(T)
-        else:
-            assert ishom2(T, check=True)
+            T = base.r2t(T)
+        elif not ishom2(T, check=True):
+            raise ValueError("argument is not valid SE(2) matrix")
 
         if axes is None:
             # create an axes
@@ -615,8 +684,8 @@ if _matplotlib_exists:
         """
         Animate a 2D coordinate frame
 
-        :param T: an SO(2) or SE(2) pose to be displayed as coordinate frame
-        :type: numpy.ndarray, shape=(2,2) or (3,3)
+        :param T: an SE(2) or SO(2) pose to be displayed as coordinate frame
+        :type: ndarray(3,3) or ndarray(2,2)
         :param nframes: number of steps in the animation [defaault 100]
         :type nframes: int
         :param repeat: animate in endless loop [default False]
@@ -637,7 +706,7 @@ if _matplotlib_exists:
              tranimate2(transl(1,2)@trot2(1), frame='A', arrow=False, dims=[0, 5])
              tranimate2(transl(1,2)@trot2(1), frame='A', arrow=False, dims=[0, 5], movie='spin.mp4')
         """
-        anim = animate.Animate2(**kwargs)
+        anim = base.animate.Animate2(**kwargs)
         anim.trplot2(T, **kwargs)
         anim.run(**kwargs)
 
@@ -650,4 +719,4 @@ if __name__ == '__main__':  # pragma: no cover
     # trplot2( transl2(4, 3)@trot2(math.pi/3), color='green', frame='c')
     # plt.grid(True)
 
-    exec(open(pathlib.Path(__file__).parent.absolute() / "test_transforms.py").read())  # pylint: disable=exec-used
+    exec(open(pathlib.Path(__file__).parent.absolute() / "test_transforms2d.py").read())  # pylint: disable=exec-used
