@@ -1,8 +1,11 @@
+# Part of Spatial Math Toolbox for Python
+# Copyright (c) 2000 Peter Corke
+# MIT Licence, see details in top-level file: LICENCE
+
 import numpy as np
-import math
 
 from spatialmath.pose3d import SO3, SE3
-from spatialmath.pose2d import SO2, SE2
+from spatialmath.pose2d import SE2
 from spatialmath.geom3d import Plucker
 import spatialmath.base as base
 from spatialmath.smuserlist import SMUserList
@@ -133,6 +136,18 @@ class SMTwist(SMUserList):
         else:
             return [base.isunittwist(x) for x in self.data]
 
+    def inv(self):
+        """
+        Inverse of Twist (superclass method)
+
+        :return: inverse
+        :rtype: Twist instance
+
+        Compute the inverse of each of the values within the twist instance.
+        The inverse is the negative of the twist vector.
+        """
+        return self.__class__([-t for t in self.data])
+
 
 # ======================================================================== #
 
@@ -544,11 +559,11 @@ class Twist3(SMTwist):
             # return Twist(left.S * right)
             return Twist3(left.binop(right, lambda x, y: x * y))
         elif isinstance(right, SpatialVelocity):
-            return SpatialVelocity(left.Ad @ right.V)
+            return SpatialVelocity(left.Ad() @ right.A)
         elif isinstance(right, SpatialAcceleration):
-            return SpatialAcceleration(left.Ad @ right.V)
+            return SpatialAcceleration(left.Ad() @ right.A)
         elif isinstance(right, SpatialForce):
-            return SpatialForce(left.Ad @ right.V)
+            return SpatialForce(left.Ad().T @ right.A)
         else:
             raise ValueError('twist *, incorrect right operand')
 
